/*
 * @(#)52    1.2  src/pdwebrte/azn/eaidemo/EAIDemo/JavaSource/com/tivoli/am/eb/web/eaidemo/AdvancedEAIDemoServlet.java, pdweb.ws.authn.eai, amwebrte700  9/27/04  19:07:07
 * ===========================================================================
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2004 All Rights Reserved.
 * ===========================================================================
 *
 */
package com.tivoli.am.eb.web.eaidemo;


import com.tivoli.pd.jazn.PDAuthorizationContext;
import com.tivoli.pd.jazn.PDPrincipal;
import com.tivoli.pd.jutil.PDAttrValue;
import com.tivoli.pd.jutil.PDAttrValueList;
import com.tivoli.pd.jutil.PDException;

import java.net.URL;

import java.util.ArrayList;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 * The advanced EAI demo servlet performs authentication through the TAM Java
 * Authentication API by generating a PAC for the user corresponding to the
 * <code>tam-eai-user</code> servlet context parameter.  The user ID entered
 * in the EAI login form is set as an extended attribute and returned in the
 * response headers for requests made of this servlet.  In order for this
 * servlet to function correctly, the location of
 * <code>PdPerm.properties</code> must be set in the <code>jrte-cfg-url</code>
 * servlet context parameter. Additionally, the JRE must be configured for use
 * with TAM by running PdJrteCfg and SvrSslCfg. See the WebSphere
 * documentation for enabling use with Tivoli Access Manager for the proper
 * usage of the PdJrteCfg and SvrSslCfg commands.
 *
 * @author Tivoli Systems
 */
public class AdvancedEAIDemoServlet extends AbstractEAIDemoServlet {
    private PDAuthorizationContext ctx = null;


    /**
     * Constructs a new advanced demo servlet.
     */
    public AdvancedEAIDemoServlet() {
    }


    /**
     * Initializes the PDAuthorizationContext, preparing the servlet for
     * authentication requests.
     *
     * @throws ServletException if the server is misconfigured
     */
    public void init() throws ServletException {
        super.init();

        // make sure the PAC header name has been specified
        if(config.getEAIPACHeader() == null) {
            throw new ServletException("Please enable the eai-pac-header parameter.");
        }

        // get the URL of the PdPerm.properties file
        URL propsUrl = null;

        try {
            propsUrl = config.getPDPermPropertiesURL();
            ctx = new PDAuthorizationContext(propsUrl);
        } catch(Exception ex) {
            throw new ServletException(ex);
        }
    }


    /**
     * Performs authentication against a the registry configured in the PDJRTE
     * for this server. All user IDs entered into the EAI login form are
     * mapped to a common TAM user ID. This user ID can be configured by
     * changing the context parameter <code>tam-eai-user</code>. The user ID
     * mapping is implemented through EAI extended attributes.
     *
     * @param req The HTTP request containing login data from the EAI login
     *        form.
     *
     * @return A Principal (of type
     *         <code>com.tivoli.pd.jazn.PDPrincipal</code>) generated from the
     *         PDJRTE. This method will return null if no login data was
     *         found.
     *
     * @throws Exception If there was a problem authenticating the user through
     *         the PDJRTE.
     */
    protected Object doAuthentication(HttpServletRequest req)
                               throws Exception
    {
        PDPrincipal principal = null;

        // establish an authorization context and get the principal                
        if((getDeclaredTamUser() == null) ||
               getDeclaredTamUser().equalsIgnoreCase("unauthenticated")) {
            principal = new PDPrincipal(ctx, config.getDefaultUser());
        } else {
            principal = new PDPrincipal(ctx, getDeclaredTamUser());
        }

        String loginFormUsername = req.getParameter("usernamefield");
        
        if(loginFormUsername != null) {
            // add the name of the header whose value corresponds to the
            //  original user ID entered in the EAI login form
            ArrayList list = new ArrayList();

            list.add(new PDAttrValue(ctx, loginFormUsername));

            PDAttrValueList attrs = new PDAttrValueList(ctx, list);

            PDPrincipal temp =
                principal.addAttribute(ctx, ORIG_USER_ATTR_NAME, attrs);

            if(temp != null) {
                principal = temp;
            }
        }
        
        // set the authentication level by adding it as an attribute to the
        //  principal
        String authLevel = null;
        if(getDeclaredAuthLevel() != null) {
            authLevel = getDeclaredAuthLevel();
        } else {
            authLevel = config.getDefaultAuthLevelValue();
        }
        
        if(authLevel != null) {
            ArrayList list = new ArrayList();
            list.add(new PDAttrValue(ctx, authLevel));
            PDAttrValueList attrs = new PDAttrValueList(ctx, list);
            PDPrincipal temp = 
                principal.addAttribute(ctx, "AUTHENTICATION_LEVEL", attrs);
            
            if(temp != null) {
                principal = temp;
            }
        }

        return principal;
    }


    /**
     * Sets the appropriate user ID headers in the response to the client.
     * Headers set will be: <code>eai-pac</code>, and
     * <code>eai-redir-url</code>.  If a header is not configured in the
     * web.xml file for this application, it will not be included in the
     * response.
     *
     * @param principal The user information returned from doAuthentication()
     * @param resp The HTTP response that will be sent to the client
     */
    protected void doEAIHeaders(Object principal, HttpServletResponse resp) {
        PDPrincipal p = (PDPrincipal)principal;
        
        // convert the Principal to a PAC and place the PAC in the header
        if(config.getEAIPACHeader() != null) {
            try {
                resp.setHeader(config.getEAIPACHeader(),
                               PACAdapter.PACToString(p.getPAC(ctx)));
            } catch(PDException e) {
                e.printStackTrace();
            }
        }
    }
}
