/*
 * @(#)57    1.3  src/pdwebrte/azn/eaidemo/EAIDemo/JavaSource/com/tivoli/am/eb/web/eaidemo/SimpleEAIDemoServlet.java, pdweb.ws.authn.eai, amwebrte700  8/2/05  11:27:44
 * ===========================================================================
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2005 All Rights Reserved.
 * ===========================================================================
 *
 */
package com.tivoli.am.eb.web.eaidemo;


import javax.servlet.Servlet;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 * A demo EAI servlet that returns the EAI user ID headers.
 *
 * @author Tivoli Systems
 */
public class SimpleEAIDemoServlet extends AbstractEAIDemoServlet
    implements Servlet
{
    /* (non-Javadoc)
     * @see javax.servlet.GenericServlet#init()
     */
    public void init() throws ServletException {
        super.init();

        // make sure the User ID header has been specified
        if(config.getEAIUserIDHeader() == null) {
            throw new ServletException("Please enable the eai-user-id-header parameter.");
        } else if(config.getEAIAuthLevelHeader() == null) {
            throw new ServletException("Please enable the eai-auth-level parameter.");
        }
    }


    /**
     * Locates the username submitted from the EAI login form.
     *
     * @param req The HTTP request containing the login data from the EAI login
     *        form.
     *
     * @return The username submitted from the EAI login form.
     */
    protected Object doAuthentication(HttpServletRequest req) {
        return req.getParameter("usernamefield");
    }


    /**
     * Sets the appropriate user ID headers in the response to the client.
     * Headers set will be: <code>eai-user-id</code>,
     * <code>eai-auth-level</code>, <code>eai-xattrs</code>, and
     * <code>eai-redir-url</code>.    If a header is not configured in the
     * <code>web.xml</code> file for this  application, it will not be
     * included in the response. This method will  set the user ID typed into
     * the EAI login form as an extended attribute.  The actual user ID
     * returned can be configred in the <code>web.xml</code> file, and must
     * match a user ID that exists in the TAM registry.
     *
     * @param principal The user information returned from doAuthentication()
     * @param resp The HTTP response that will be sent to the client
     */
    protected void doEAIHeaders(Object principal, HttpServletResponse resp) {
        String user = (String)principal;

        // set the user id header. Note: user id must alway be retrieved from 
        // eaidemo.properties because username may be changed during the reauth.
        resp.setHeader(config.getEAIUserIDHeader(), config.getDefaultUser());

        // set the extended attributes and original user name headers
        if(config.getEAIxattrsHeader() != null) {
            resp.setHeader(config.getEAIxattrsHeader(), ORIG_USER_ATTR_NAME);
            resp.setHeader(ORIG_USER_ATTR_NAME, user);
        }
        
        // set the auth level header
        if(getDeclaredAuthLevel() != null) {
            resp.setHeader(config.getEAIAuthLevelHeader(),
                           getDeclaredAuthLevel());
        } else {
            resp.setHeader(config.getEAIAuthLevelHeader(),
                           config.getDefaultAuthLevelValue());
        }        
    }
}
